Option Explicit

Class HandlerClass
 Private fWarning
'---Declarations for expecting events
 ' fExpect flags are automatically set and cleared when aco's or coacos are added
 ' adding another aco or coaco in the same expect will reset the handler and ignore previous expectations
 ' when an item is moved in inventory, both events fire
 Public fExpectMove, fMoveComplete, fExpectAdd, fAddComplete
 ' only 1 item moves at a time, but multiple adds can be expected when dealing with vendors
 Public acoExpectedMove, dExpectedAdd, szExpectedAdd
 ' for dealing with pyreals from vendors
 Public iExpectPyreals, iPriorPyreals, fExpectPyreals, fPyrealsAdded
 
 
 ' items can be removed under many circumstances and multiple removes are common
 Public fExpectRemove, fRemoveComplete
 Public coExpectedRemove
 ' only 2 stacks can adjust at a time
 Public fExpectAdjustA, fAdjustCompleteA, fExpectAdjustB, fAdjustCompleteB
 Public acoExpectedAdjustA, acoExpectedAdjustB
 '
 Public fExpectCraft, fCraftComplete, fCraftSuccess
 
 Public fExpectVendor, fVendorOpen
 Public acoExpectedVendor
 Public mLastMerchantCoaco
 Public mLastMerchantMarkup
'---

'--- Events for Crafting
 Public Function OnChatServer (szText, cmcColor)
  If fExpectCraft And cmcColor = cmcGreen And "You " = Left(szText,4) Then
  	fExpectCraft = False
  	fCraftComplete = True
  	If "You fail" = Left(szText,8) Then
  		fCraftSuccess = False
  	Else
  		fCraftSuccess = True
  	End If
  End If
 End Function
 Public Sub ExpectCraft ()
  fExpectCraft = True
  fCraftComplete = False
 End Sub
 Public Property Get CraftComplete ()
  CraftComplete = fCraftComplete
 End Property
 Public Property Get CraftSuccess ()
  CraftSuccess = fCraftSuccess
 End Property
'--- End Crafting Events

'--- Events for vendors
 Public Function OnApproachVendor (acoMerchant, mcmBuy, cpyMaxBuy, fractBuy, fractSell, coItems)
  mLastMerchantMarkup = fractSell
  Set mLastMerchantCoaco = coItems
  If Not acoExpectedVendor Is Nothing Then
  	If acoMerchant Is acoExpectedVendor Then
  		fExpectVendor = False
  		Set acoExpectedVendor = Nothing
  	End If
  End If
  fVendorOpen = True
 End Function
 Public Property Let ExpectVendor (acoNew)
  If IsObject(acoNew) Then
  	fExpectVendor = True
  	fVendorOpen = False
  	Set acoExpectedVendor = acoNew
  End If
 End Property
 Public Property Get VendorOpen ()
  VendorOpen = fVendorOpen
 End Property
 Public Property Get LastMerchantCoaco()
  Set LastMerchantCoaco = mLastMerchantCoaco
 End Property
 Public Property Get LastMerchantMarkup()
  LastMerchantMarkup = mLastMerchantMarkup
 End Property
'--- End Vendor Events

'--- Events that throw warnings
 Public Function OnPortalStormed ()
  fWarning = True
 End Function
 Public Function OnEnd3D ()
  fWarning = True
 End Function
 Public Function OnDeathSelf (acoKiller, szDeathMsg)
  Computer.Out "OMGWTFBBQ? Killed by " & acoKiller.szName & "."
  fWarning = True
 End Function
 Public Function OnMeleeDamageSelf (szName, dhealth, dmty, bpart)
  Computer.Out "Attacked by " & szName & "."
  fWarning = True
 End Function
 Public Function OnDisconnect ()
  Computer.Out "Disconnected from game server."
  fWarning = True
 End Function
 Public Property Get Warning ()
  Warning = fWarning
 End Property
'--- End Warning Events


'--- Inventory Events 
 Public Function OnRemoveFromInventory(acoRemoved, acoContainer, iItem)
  If coExpectedRemove Is Nothing Then Exit Function
  If fExpectRemove And coExpectedRemove.FContainsAco(acoRemoved) Then
  	coExpectedRemove.RemoveAco(acoRemoved)
  	If coExpectedRemove.Count = 0 Then
  		fRemoveComplete = True
  		Set coExpectedRemove = Nothing
  	End If
  End If
 End Function
 Public Property Let ExpectRemove (coNew)
  If coNew.Count > 0 Then
  	fExpectRemove = True
  	fRemoveComplete = False
  	Set coExpectedRemove = coNew
  End If
 End Property
 Public Property Get RemoveComplete ()
  RemoveComplete = fRemoveComplete
 End Property
 
 Public Function OnMoveItem (acoItem, acoCont, iItem)
  If acoExpectedMove Is Nothing Then Exit Function
  If fExpectMove And acoExpectedMove Is acoItem Then
  	fExpectMove = False
  	fMoveComplete = True
  	Set acoExpectedMove = Nothing
  End If
 End Function
 Public Property Let ExpectMove(acoNew)
  If IsObject(acoNew) Then
  	fMoveComplete = False
  	fExpectMove = True
  	Set acoExpectedMove = acoNew
  Else
  	Computer.Out "ERROR in ExpectMove: Argument provided NOT and object!"
  End If
 End Property
 Public Property Get MoveComplete()
  MoveComplete = fMoveComplete
 End Property

 Public Function OnAdjustStack (acoStack, cItemPrev)
  If acoExpectedAdjustA Is Nothing And acoExpectedAdjustB Is Nothing Then
  	Exit Function
  End If
  If Not acoExpectedAdjustA Is Nothing Then
  	If fExpectAdjustA And acoExpectedAdjustA Is acoStack Then
  		fExpectAdjustA = False
  		fAdjustCompleteA = True
  		Set acoExpectedAdjustA = Nothing
  	End If
  End If
  If acoExpectedAdjustB Is Nothing Then
  	Exit Function
  Else
  	If fExpectAdjustB And acoExpectedAdjustB Is acoStack Then
  		fExpectAdjustB = False
  		fAdjustCompleteB = True
  		Set acoExpectedAdjustB = Nothing  	
  	End If
  End If
 End Function
 Public Sub ExpectAdjust(acoOne, acoTwo)
  If IsObject(acoOne) Then
  	fAdjustCompleteA = False
  	fExpectAdjustA = True
  	Set acoExpectedAdjustA = acoOne
  	If IsObject(acoTwo) Then
  		fAdjustCompleteB = False
  		fExpectAdjustB = True
  		Set acoExpectedAdjustB = acoTwo
  	End If
  Else
  	Computer.Out "ERROR in ExpectAdjust: First argument provided NOT and object!"
  End If
 End Sub
 Public Property Get AdjustComplete
  If (Not acoExpectedAdjustB Is Nothing) And fAdjustCompleteA And fAdjustCompleteB Then
  	AdjustComplete = True
  End If
  If acoExpectedAdjustB Is Nothing And fAdjustCompleteA Then
  	AdjustComplete = True
  End If
 End Property
 
 Public Function OnAddToInventory (acoAdded)
  If fExpectPyreals And skapi.cpycash - iPriorPyreals = iExpectPyreals Then
  	fExpectPyreals = False
  	fPyrealsAdded = True
  	iExpectPyreals = 0
  End If
  If szExpectedAdd = acoAdded.szName Then
  	fExpectAdd = False
  	fAddComplete = True
  	szExpectedAdd = ""
  	Exit Function
  End If
  If dExpectedAdd.Count = 0 Then Exit Function
  'Computer.Out "Passed dictionary lock"
  If fExpectAdd And dExpectedAdd.Exists(acoAdded.szName) Then
  	'Computer.Out "Passed second dictionary lock."
  	If dExpectedAdd.Item(acoAdded.szName) > 1 Then
  		dExpectedAdd.Item(acoAdded.szName) = dExpectedAdd.Item(acoAdded.szName) - 1
  		'Computer.Out "Have one of several items."
  	Else
  		Call dExpectedAdd.Remove(acoAdded.szName)
  		'Computer.Out "Have only or final item."
  	End If
  	If dExpectedAdd.Count = 0 Then
  		fExpectAdd = False
  		fAddComplete = True
  	End If
  End If
 End Function
 Public Sub ExpectPyreals(iPyreals)
  iExpectPyreals = iPyreals
  iPriorPyreals = skapi.cpycash
  fExpectPyreals = True
  fPyrealsAdded = False
 End Sub
 Public Sub ExpectAddsz (szNew)
  fExpectAdd = True
  fAddComplete = False
  szExpectedAdd = szNew
 End Sub
 Public Sub ExpectAdd(szNew, iCount)
  If Not dExpectedAdd.Exists(szNew) Then
  	'Computer.Out "Adding new entry."
  	Call dExpectedAdd.Add(szNew, iCount)
  Else
  	'Computer.Out "Adding more to existing entry."
  	dExpectedAdd.Item(szNew) = dExpectedAdd.Item(szNew) + iCount
  End If
  fExpectAdd = True
  fAddComplete = False
 End Sub
 Public Sub ResetExpectAdd ()
  'Computer.Out "Resetting add expectation."
  Set dExpectedAdd = CreateObject("Scripting.Dictionary")
  fExpectAdd = False
  fAddComplete = False
 End Sub
 Public Property Get AddComplete()
  AddComplete = fAddComplete
 End Property
 Public Property Get PyrealsAdded()
  PyrealsAdded = fPyrealsAdded
 End Property
'--- End Inventory Events

 Private Sub Class_Initialize()
  Computer.Out "Handler Initializing."
  fWarning = False
  fExpectCraft = False
  fCraftComplete = False
  fExpectMove = False
  fMoveComplete = False
  fExpectAdd = False
  iExpectPyreals = 0
  iPriorPyreals = 0
  fExpectPyreals = False
  fPyrealsAdded = False
  fAddComplete = False
  fExpectRemove = False
  fRemoveComplete = False
  fExpectAdjustA = False
  fAdjustCompleteA = False
  fExpectAdjustB = False
  fAdjustCompleteB = False
  Set acoExpectedMove = Nothing
  Set dExpectedAdd = CreateObject("Scripting.Dictionary")
  Set coExpectedRemove = Nothing
  Set acoExpectedAdjustA = Nothing
  Set acoExpectedAdjustB = Nothing
 End Sub
 
 Public Sub Register ()
  Call skapi.AddHandler(evidOnDisconnect, Handler)
  Call skapi.AddHandler(evidOnPortalStormed, Handler)
  Call skapi.AddHandler(evidOnDeathSelf, Handler)
  Call skapi.AddHandler(evidOnMeleeDamageSelf, Handler)
  Call skapi.AddHandler(evidOnEnd3d, Handler)
  Call skapi.AddHandler(evidOnAddToInventory, Handler)
  Call skapi.AddHandler(evidOnAdjustStack, Handler)
  Call skapi.AddHandler(evidOnMoveItem, Handler)
  Call skapi.AddHandler(evidOnRemoveFromInventory, Handler)
  Call skapi.AddHandler(evidOnApproachVendor, Handler)
  Call skapi.AddHandler(evidOnChatServer, Handler)
 End Sub  
 
 Private Sub Class_Terminate()
  Computer.Out "Handler Terminating."
  Call skapi.RemoveHandler(evidOnDisconnect, Handler)
  Call skapi.RemoveHandler(evidOnPortalStormed, Handler)
  Call skapi.RemoveHandler(evidOnDeathSelf, Handler)
  Call skapi.RemoveHandler(evidOnMeleeDamageSelf, Handler)
  Call skapi.RemoveHandler(evidOnEnd3d, Handler)
  Call skapi.RemoveHandler(evidOnAddToInventory, Handler)
  Call skapi.RemoveHandler(evidOnAdjustStack, Handler)
  Call skapi.RemoveHandler(evidOnMoveItem, Handler)
  Call skapi.RemoveHandler(evidOnRemoveFromInventory, Handler)
  Call skapi.RemoveHandler(evidOnApproachVendor, Handler)
  Call skapi.RemoveHandler(evidOnChatServer, Handler)
 End Sub
 

End Class
